@@ -1,11 +1,9 @@
-import os
-import re
 import shutil
 import time
 import traceback
 from subprocess import Popen, STDOUT
 from threading import Thread
-from typing import List, Set, Type, Tuple, Dict, Optional, Generator, Callable, Pattern
+from typing import List, Set, Type, Tuple, Dict, Optional, Generator, Callable
 
 from bauh.api.abstract.controller import SoftwareManager, SearchResult, ApplicationContext, UpgradeRequirements, \
     UpgradeRequirement, TransactionResult, SoftwareAction, SettingsView, SettingsController
@@ -13,10 +11,11 @@ from bauh.api.abstract.disk import DiskCacheLoader
 from bauh.api.abstract.handler import ProcessWatcher, TaskManager
 from bauh.api.abstract.model import SoftwarePackage, PackageUpdate, PackageHistory, PackageSuggestion, \
     CustomSoftwareAction
-from bauh.api.abstract.view import ViewComponent, TabGroupComponent, MessageType, PanelComponent
+from bauh.api.abstract.view import TabGroupComponent, MessageType
 from bauh.api.exception import NoInternetException
 from bauh.commons.boot import CreateConfigFile
 from bauh.commons.html import bold
+from bauh.commons.regex import RE_URL
 from bauh.commons.util import sanitize_command_input
 from bauh.view.core.config import CoreConfigManager
 from bauh.view.core.settings import GenericSettingsManager
@@ -25,8 +24,6 @@ from bauh.view.util import resource
 from bauh.view.util.resource import get_path
 from bauh.view.util.util import clean_app_files, restart_app
 
-RE_IS_URL = re.compile(r'^https?://.+')
-
 
 class GenericUpgradeRequirements(UpgradeRequirements):
 
@@ -144,14 +141,13 @@ class GenericSoftwareManager(SoftwareManager, SettingsController):
         return available
 
     def _search(self, word: str, is_url: bool, man: SoftwareManager, disk_loader, res: SearchResult):
-        if self._can_work(man):
-            mti = time.time()
-            apps_found = man.search(words=word, disk_loader=disk_loader, is_url=is_url, limit=-1)
-            mtf = time.time()
-            self.logger.info(f'{man.__class__.__name__} took {mtf - mti:.8f} seconds')
+        mti = time.time()
+        apps_found = man.search(words=word, disk_loader=disk_loader, is_url=is_url, limit=-1)
+        mtf = time.time()
+        self.logger.info(f'{man.__class__.__name__} took {mtf - mti:.8f} seconds')
 
-            res.installed.extend(apps_found.installed)
-            res.new.extend(apps_found.new)
+        res.installed.extend(apps_found.installed)
+        res.new.extend(apps_found.new)
 
     def search(self, words: str, disk_loader: DiskCacheLoader = None, limit: int = -1, is_url: bool = False) -> SearchResult:
         ti = time.time()
@@ -164,16 +160,17 @@ class GenericSoftwareManager(SoftwareManager, SettingsController):
             self.logger.info(f"Search query: {norm_query}")
 
             if norm_query:
-                is_url = bool(RE_IS_URL.match(norm_query))
+                is_url = bool(RE_URL.match(norm_query))
                 disk_loader = self.disk_loader_factory.new()
                 disk_loader.start()
 
                 threads = []
 
                 for man in self.managers:
-                    t = Thread(target=self._search, args=(norm_query, is_url, man, disk_loader, res))
-                    t.start()
-                    threads.append(t)
+                    if self._can_work(man):
+                        t = Thread(target=self._search, args=(norm_query, is_url, man, disk_loader, res), daemon=True)
+                        t.start()
+                        threads.append(t)
 
                 for t in threads:
                     t.join()
@@ -181,9 +178,6 @@ class GenericSoftwareManager(SoftwareManager, SettingsController):
                 if disk_loader:
                     disk_loader.stop_working()
                     disk_loader.join()
-
-            # res.installed = self._sort(res.installed, norm_word)
-            # res.new = self._sort(res.new, norm_word)
         else:
             raise NoInternetException()
 
@@ -206,6 +200,15 @@ class GenericSoftwareManager(SoftwareManager, SettingsController):
     def _get_package_lower_name(self, pkg: SoftwarePackage):
         return pkg.name.lower()
 
+    def _fill_read_installed(self, man: SoftwareManager, disk_loader: DiskCacheLoader, internet_available: bool,
+                             output: List[SearchResult]):
+        mti = time.time()
+        man_res = man.read_installed(disk_loader=disk_loader, pkg_types=None, internet_available=internet_available,
+                                     limit=-1, only_apps=False)
+        mtf = time.time()
+        self.logger.info(f'{man.__class__.__name__} took {mtf - mti:.4f} seconds')
+        output.append(man_res)
+
     def read_installed(self, disk_loader: DiskCacheLoader = None, limit: int = -1, only_apps: bool = False, pkg_types: Set[Type[SoftwarePackage]] = None, internet_available: bool = None) -> SearchResult:
         ti = time.time()
         self._wait_to_be_ready()
@@ -215,6 +218,9 @@ class GenericSoftwareManager(SoftwareManager, SettingsController):
         disk_loader = None
 
         net_available = self.context.is_internet_available()
+        read_threads = list()
+        results = list()
+
         if not pkg_types:  # any type
             for man in self.managers:
                 if self._can_work(man):
@@ -222,13 +228,11 @@ class GenericSoftwareManager(SoftwareManager, SettingsController):
                         disk_loader = self.disk_loader_factory.new()
                         disk_loader.start()
 
-                    mti = time.time()
-                    man_res = man.read_installed(disk_loader=disk_loader, pkg_types=None, internet_available=net_available)
-                    mtf = time.time()
-                    self.logger.info(f'{man.__class__.__name__} took {mtf - mti:.2f} seconds')
-
-                    res.installed.extend(man_res.installed)
-                    res.total += man_res.total
+                    t = Thread(target=self._fill_read_installed,
+                               args=(man, disk_loader, net_available, results),
+                               daemon=True)
+                    t.start()
+                    read_threads.append(t)
         else:
             man_already_used = []
 
@@ -240,18 +244,24 @@ class GenericSoftwareManager(SoftwareManager, SettingsController):
                         disk_loader = self.disk_loader_factory.new()
                         disk_loader.start()
 
-                    mti = time.time()
-                    man_res = man.read_installed(disk_loader=disk_loader, pkg_types=None, internet_available=net_available)
-                    mtf = time.time()
-                    self.logger.info(f'{man.__class__.__name__} took {mtf - mti:.2f} seconds')
+                    t = Thread(target=self._fill_read_installed,
+                               args=(man, disk_loader, net_available, results),
+                               daemon=True)
+                    t.start()
+                    read_threads.append(t)
 
-                    res.installed.extend(man_res.installed)
-                    res.total += man_res.total
+        for t in read_threads:
+            t.join()
 
         if disk_loader:
             disk_loader.stop_working()
             disk_loader.join()
 
+        for result in results:
+            if result.installed:
+                res.installed.extend(result.installed)
+                res.total += result.total
+
         if res.installed:
             for p in res.installed:
                 if p.is_update_ignored():
@@ -498,7 +508,8 @@ class GenericSoftwareManager(SoftwareManager, SettingsController):
                 suggestions, threads = [], []
                 for man in self.managers:
                     t = Thread(target=self._fill_suggestions,
-                               args=(suggestions, man, int(self.config['suggestions']['by_type']), filter_installed))
+                               args=(suggestions, man, int(self.config['suggestions']['by_type']), filter_installed),
+                               daemon=True)
                     t.start()
                     threads.append(t)
 
